#!/usr/bin/perl -I/usr/sausalito/perl -I.
# $Id: writeback_blueOnyx_conf.pl
#
# This handler is responsible for updating /etc/httpd/conf.d/blueonyx.conf,
# as well as writing /var/www/html/robots.txt and /var/www/html/robots_deny_all.txt
#

my $confdir = '/etc/httpd/conf.d';
my $blueonyx_conf = "$confdir/blueonyx.conf";

my $robots_txt = '/var/www/html/robots.txt';
my $robots_deny_all_txt = '/var/www/html/robots_deny_all.txt';

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
    use Data::Dumper;
    &debug_msg("Debugging enabled for writeback_blueOnyx_conf.pl\n");
}

use Sauce::Util;
use CCE;

my $cce = new CCE;
$cce->connectfd();

my ($oid) = $cce->find("System");
my ($ok, $obj) = $cce->get($oid);
my ($status, $web) = $cce->get($oid, "Web");

if (!Sauce::Util::editfile(($blueonyx_conf),*edit_blueonyx, $web)) {
    $cce->bye('FAIL', '[[base-apache.cantEdit_BlueOnyx_conf]]');
    exit(1);
}

# Update robots.txt based on good and bad user-agents
if (!update_robots_txt($web->{'good_useragents'}, $web->{'bad_useragents'}, $cce)) {
    $cce->bye('FAIL', '[[base-apache.cantEdit_robots_txt]]');
    exit(1);
}

$cce->bye('SUCCESS');
exit(0);

#
### Subs:
#

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

sub update_robots_txt {
    my ($good_useragents, $bad_useragents) = @_;

    my @good_agents = $cce->scalar_to_array($good_useragents);
    my @bad_agents = $cce->scalar_to_array($bad_useragents);

    open(my $fh, '>', $robots_txt) or return 0;

    print $fh "# Generated robots.txt by BlueOnyx\n\n";

    # Allow well-known search engines and reputable crawlers
    print $fh "# Allow well-known search engines and reputable crawlers\n";
    foreach my $ua (@good_agents) {
        print $fh "User-agent: $ua\n";
        print $fh "Allow: /\n\n";
    }

    # Block known unwanted bots and scrapers
    print $fh "# Block known unwanted bots and scrapers\n";
    foreach my $ua (@bad_agents) {
        print $fh "User-agent: $ua\n";
        print $fh "Disallow: /\n\n";
    }

    # General restrictions for all other bots
    print $fh "# General restrictions for all other bots\n";
    print $fh "User-agent: *\n";
    print $fh "Disallow: /cgi-bin/\n";
    print $fh "Disallow: /wp-admin/\n";
    print $fh "Disallow: /wp-includes/\n";
    print $fh "Disallow: /tmp/\n";
    print $fh "Disallow: /private/\n";
    print $fh "Disallow: /config/\n";

    close($fh);

    open(my $fh, '>', $robots_deny_all_txt) or return 0;
    print $fh "# Generated robots_deny_all.txt by BlueOnyx\n\n";
    print $fh "User-agent: *\n";
    print $fh "Disallow: /\n";
    print $fh "\n";
    close($fh);

    return 1;
}

sub edit_blueonyx {
    my ($in, $out, $webdata) = @_;
    my $begin = '<Directory /home/.sites/>';
    my $script_conf = '';

    # Debugging output for troubleshooting
    &debug_msg("\$webdata->{Options_All}: $webdata->{Options_All}");
    &debug_msg("\$webdata->{Options_FollowSymLinks}: $webdata->{Options_FollowSymLinks}");
    &debug_msg("\$webdata->{Options_Includes}: $webdata->{Options_Includes}");
    &debug_msg("\$webdata->{Options_Indexes}: $webdata->{Options_Indexes}");
    &debug_msg("\$webdata->{Options_MultiViews}: $webdata->{Options_MultiViews}");
    &debug_msg("\$webdata->{Options_SymLinksIfOwnerMatch}: $webdata->{Options_SymLinksIfOwnerMatch}");

    &debug_msg("\$webdata->{AllowOverride_All}: $webdata->{AllowOverride_All}");
    &debug_msg("\$webdata->{AllowOverride_AuthConfig}: $webdata->{AllowOverride_AuthConfig}");
    &debug_msg("\$webdata->{AllowOverride_FileInfo}: $webdata->{AllowOverride_FileInfo}");
    &debug_msg("\$webdata->{AllowOverride_Indexes}: $webdata->{AllowOverride_Indexes}");
    &debug_msg("\$webdata->{AllowOverride_Limit}: $webdata->{AllowOverride_Limit}");
    &debug_msg("\$webdata->{AllowOverride_Options}: $webdata->{AllowOverride_Options}");

    # Collect Options based on configuration
    my @Options;

    # Always enforce -Indexes unless explicitly enabled
    if ($webdata->{Options_All}) {
        push(@Options, "All");
    } 
    else {
        push(@Options, $webdata->{Options_Indexes} ? "+Indexes" : "-Indexes");
        push(@Options, "+SymLinksIfOwnerMatch") if $webdata->{Options_SymLinksIfOwnerMatch};
        push(@Options, "+FollowSymLinks") if $webdata->{Options_FollowSymLinks};
        push(@Options, "+Includes") if $webdata->{Options_Includes};
        push(@Options, "+MultiViews") if $webdata->{Options_MultiViews};
    }

    # Enforce -Indexes if AllowOverride_Indexes is disabled
    if ($webdata->{Options_Indexes}) {
        push(@Options, "+Indexes");
    }
    else {
        push(@Options, "-Indexes");
    }

    # Collect AllowOverride settings
    my @AllowOverride;
    push(@AllowOverride, "AuthConfig") if $webdata->{AllowOverride_AuthConfig};
    push(@AllowOverride, "Indexes") if $webdata->{AllowOverride_Indexes};
    push(@AllowOverride, "Limit") if $webdata->{AllowOverride_Limit};
    push(@AllowOverride, "FileInfo") if $webdata->{AllowOverride_FileInfo};

    # Build Options= part for AllowOverride, including MultiViews if enabled
    if ($webdata->{AllowOverride_Options}) {
        my @allowed_options;
        push(@allowed_options, "MultiViews") if $webdata->{Options_MultiViews};
        push(@allowed_options, "SymLinksIfOwnerMatch") if $webdata->{Options_SymLinksIfOwnerMatch};
        push(@allowed_options, "FollowSymLinks") if $webdata->{Options_FollowSymLinks};
        push(@allowed_options, "Includes") if $webdata->{Options_Includes};

        # Handle Indexes properly based on AllowOverride_Indexes
        if ($webdata->{AllowOverride_Indexes}) {
            push(@allowed_options, "Indexes");
        }
        else {
            push(@allowed_options, "-Indexes");
        }

        # Handle SymLinksIfOwnerMatch properly:
        if ($webdata->{Options_SymLinksIfOwnerMatch}) {
            push(@allowed_options, "SymLinksIfOwnerMatch");
        }
        else {
            push(@allowed_options, "-SymLinksIfOwnerMatch");
        }

        # Always generate Options= even if Options_All is enabled
        if (@allowed_options) {
            my $options_string = join(",", @allowed_options);
            push(@AllowOverride, "Options=$options_string");
            &debug_msg("Generated Options= string: Options=$options_string");
        }
    }

    # Always append 'All' if explicitly enabled
    push(@AllowOverride, "All") if $webdata->{AllowOverride_All};

    # Generate configuration lines
    my $out_options = join(" ", @Options);
    my $out_AllowOverride = join(" ", @AllowOverride);

    &debug_msg("Generated Options: $out_options");
    &debug_msg("Generated AllowOverride: $out_AllowOverride");

    $script_conf .= "<Directory /home/.sites/>\n";
    $script_conf .= "Options $out_options\n";
    $script_conf .= "AllowOverride $out_AllowOverride\n";
    $script_conf .= "\n# ignore .ht*\n";

    # Process input and output
    my $last;
    while (<$in>) {
        if (/^Alias \/libImage\/(.*)$/) { $last = $_; last; }

        if (/^$begin$/) {
            while (<$in>) {
                last if /^# ignore .ht(.*)$/;
            }
            print $out $script_conf;
        }
        else {
            print $out $_;
        }
    }
    print $out $last if $last;

    # Preserve the remainder of the config file
    while (<$in>) {
        print $out $_;
    }

    return 1;
}

$cce->bye('SUCCESS');
exit(0);

# 
# Copyright (c) 2008-2025 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2025 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 